﻿/// <reference path="typings/c1.mvc.basic.lib.d.ts" />

class CustomCell{
    private _grid: wijmo.grid.FlexGrid;
    private _binding: string;
    private _columnIndex: number;
    private _control: object;
    private _class: any;
    private _options: object;
    constructor(grid: wijmo.grid.FlexGrid, columnIndex: number, binding: string, cellClass:any,options:object) {
        this._grid = grid;
        this._binding = binding;
        this._columnIndex = columnIndex;
        this._class = cellClass;
        this._options = options;
        this.createCellTemplate();
    }

    createCellTemplate() {
        this._grid.formatItem.addHandler((grid: wijmo.grid.FlexGrid, args: wijmo.grid.FormatItemEventArgs) => {
            if (args.panel.cellType === wijmo.grid.CellType.Cell && args.col === this._columnIndex) {
                let div = wijmo.createElement("<div style='width:50%;height:50%;'></div>");
                div.id = this._getControlId(args.row, args.col);
                this._control = new this._class(div, this._options);
                
                if ((this._control instanceof wijmo.input.ComboBox) || (this._control instanceof wijmo.input.AutoComplete)) {
                    let _ctrl = this._control as wijmo.input.ComboBox;
                    _ctrl.selectedValue = grid.rows[args.row].dataItem[this._binding];
                    _ctrl.textChanged.addHandler((s, e) => {
                        const editItem = grid.collectionView as wijmo.collections.IEditableCollectionView;
                        editItem.currentEditItem[this._binding] = s['selectedValue'];
                    })
                    _ctrl.gotFocus.addHandler((s:wijmo.input.ComboBox, e) => {
                        let rowIndex = this._getRowIndexFromControl(s.hostElement.id);
                        if (this._grid.selection.row != rowIndex) {
                            this._grid.startEditing(false, rowIndex, this._columnIndex, true);
                        }
                    })
                    _ctrl.hostElement.addEventListener("keydown", (e: KeyboardEvent) => {
                        let _nextElement = _ctrl.hostElement.nextElementSibling;
                        if (_nextElement && e.keyCode==13) {
                            wijmo.Control.getControl(_nextElement).focus();
                        }
                    }, true);
                }
                
                if (args.cell.children.length > 0) {
                    let children = args.cell.children[0];
                    if (wijmo.Control.getControl(children) === null) {
                        args.cell.removeChild(children);
                    }
                    args.cell.style.padding = '0';
                    args.cell.appendChild(div);
                }
            }
        });
        setTimeout( ()=> {
            this._grid.autoSizeRows();
        },100)
        //this._grid.startEditing(false, 0, 0, true);        
        this._grid.collectionView.currentItem = null;
        this._grid.selectionChanged.addHandler(this.selectionChanged.bind(this));
    }
    private _previousSelection;
    selectionChanged(s, e:wijmo.grid.CellRangeEventArgs) {
        if (this._grid) {
            this._grid.startEditing(false, e.row, e.col, true);
            //var _cell = this._grid.cells.getCellElement(e.row, e.col);
            //if (_cell.childElementCount > 0) {
            //    let _ctrl = wijmo.Control.getControl('#' + _cell.children[0].id);
            //    _ctrl.focus();
            //}
        } else {
            console.error("Error in grid");
        }
    }
    _getRowIndexFromControl(id:string) : number {
        let _i = id.indexOf("row");
        return parseInt(id.substr(_i + 3, 1));
    }
    _getGridId() {
        return this._grid.hostElement.id;
    }
    _getControlId(row: number, col: number) {
        return wijmo.getUniqueId(`${this._getGridId()}_row${row}_col${col}_ctrl`);
    }
    
}
   